require 'test_helper'

class TestHand < Test::Unit::TestCase
	def setup
		@duce = Ventiuna::Card.new(rank: "2", suit: "club")
		@tres = Ventiuna::Card.new(rank: "3", suit: "diamond")
		@quatro = Ventiuna::Card.new(rank: "4", suit: "heart")

		@seven_of_diamonds = Ventiuna::Card.new(rank: "7", suit: "diamond")
		@eight_of_spades = Ventiuna::Card.new(rank: "8", suit: "spade")
		@eight_of_clubs = Ventiuna::Card.new(rank: "8", suit: "club")

		@jack_of_clubs = Ventiuna::Card.new(rank: "J", suit: "club")
		@king_of_hearts = Ventiuna::Card.new(rank: "K", suit: "heart")
		@ace_of_diamonds = Ventiuna::Card.new(rank: "A", suit: "diamond")
		@ace_of_spades = Ventiuna::Card.new(rank: "A", suit: "spade")
	end

	def test_init
		assert Ventiuna::Hand.new
		assert Ventiuna::Hand.new(@duce)
		assert Ventiuna::Hand.new(@duce, @tres)
		assert hand = Ventiuna::Hand.new(@duce, @tres, @quatro)

		assert hand.cards.include?(@duce)
		assert hand.cards.include?(@tres)
		assert hand.cards.include?(@quatro)
	end

	def test_empty_hand
		assert_equal(0, Ventiuna::Hand.new.value)
	end

	def test_add_cards
		hand = Ventiuna::Hand.new
		hand << @duce
		hand << @tres
		hand << @quatro

		assert hand.cards.include?(@duce)
		assert hand.cards.include?(@tres)
		assert hand.cards.include?(@quatro)
	end

	def test_value
		hand = Ventiuna::Hand.new
		hand << @duce
		assert_equal(2, hand.value)

		hand << @king_of_hearts
		assert_equal(12, hand.value)

		hand << @ace_of_diamonds
		assert_equal(13, hand.value)

		hand << @quatro
		assert_equal(17, hand.value)

		hand << @tres
		assert_equal(20, hand.value)

		hand << @jack_of_clubs
		assert_equal(30, hand.value)
	end

	def test_value_with_soft_hand
		hand = Ventiuna::Hand.new
		hand << @ace_of_diamonds
		assert_equal(11, hand.value)

		hand << @duce
		assert_equal(13, hand.value)

		hand << @king_of_hearts
		assert_equal(13, hand.value)

		hand << @ace_of_spades
		assert_equal(14, hand.value)
	end

	def test_value_with_two_aces
		hand = Ventiuna::Hand.new
		hand << @tres
		assert_equal(3, hand.value)

		hand << @ace_of_spades
		assert_equal(14, hand.value)

		hand << @ace_of_diamonds
		assert_equal(15, hand.value)

		hand << @seven_of_diamonds
		assert_equal(12, hand.value)
	end

	def test_value_hit_on_11
		hand = Ventiuna::Hand.new
		hand.hit @tres
		assert_equal(3, hand.value)

		hand.hit @eight_of_spades
		assert_equal(11, hand.value)

		hand.hit @ace_of_diamonds
		assert_equal(12, hand.value)
	end

	def test_value_hit_on_10
		hand = Ventiuna::Hand.new
		hand.hit @eight_of_spades
		assert_equal(8, hand.value)

		hand.hit @duce
		assert_equal(10, hand.value)

		hand.hit @ace_of_diamonds
		assert_equal(21, hand.value)
	end

	def test_blackjack?
		hand = Ventiuna::Hand.new(@king_of_hearts, @ace_of_diamonds)
		assert hand.blackjack?

		hand = Ventiuna::Hand.new(@eight_of_spades, @duce, @ace_of_diamonds)
		assert !hand.blackjack?
	end

	def test_pair?
		hand = Ventiuna::Hand.new(@eight_of_spades, @eight_of_clubs)
		assert hand.pair?

		hand = Ventiuna::Hand.new(@king_of_hearts, @jack_of_clubs)
		assert hand.pair?

		hand = Ventiuna::Hand.new(@ace_of_diamonds, @ace_of_spades)
		assert hand.pair?

		hand = Ventiuna::Hand.new(@eight_of_spades, @ace_of_spades, @eight_of_clubs)
		assert !hand.pair?
	end

	def test_soft?
		hand = Ventiuna::Hand.new(@ace_of_spades, @duce)
		assert hand.soft?

		hand = Ventiuna::Hand.new(@tres, @ace_of_spades, @ace_of_diamonds)
		assert hand.soft?

		hand = Ventiuna::Hand.new(@tres, @ace_of_spades, @ace_of_diamonds, @seven_of_diamonds)
		assert !hand.soft?

		hand = Ventiuna::Hand.new(@ace_of_diamonds, @quatro)
		assert hand.soft?
		hand << @jack_of_clubs
		hand << @king_of_hearts
		assert !hand.soft?
		
	end

	def test_stand
		hand = Ventiuna::Hand.new(@king_of_hearts, @jack_of_clubs)
		hand.stand
		assert !hand.active?
	end

	def test_split
		hand = Ventiuna::Hand.new(@ace_of_diamonds, @ace_of_spades)

		assert hand.split
		assert hand.split.kind_of?(Array)
		assert_equal(2, hand.split.size)
		assert hand.split.collect{ |h| h.kind_of?(Ventiuna::Hand) }.all?
	end

	def test_bust?
		hand = Ventiuna::Hand.new(@ace_of_diamonds, @quatro)
		assert !hand.bust?

		hand << @jack_of_clubs
		assert !hand.bust?

		hand << @king_of_hearts
		assert hand.bust?
	end

	def test_to_db
		hand = Ventiuna::Hand.new(@ace_of_diamonds, @duce)
		assert_equal("13S", hand.to_db)

		hand = Ventiuna::Hand.new(@seven_of_diamonds, @tres)
		assert_equal("10", hand.to_db)

		hand = Ventiuna::Hand.new(@ace_of_diamonds, @ace_of_spades)
		assert_equal("12SP", hand.to_db)

		hand = Ventiuna::Hand.new(@jack_of_clubs, @king_of_hearts)
		assert_equal("20P", hand.to_db)
	end
end